﻿using IgOutlook.Business.Contacts;
using IgOutlook.Infrastructure;
using IgOutlook.Infrastructure.Dialogs;
using IgOutlook.Services;
using Microsoft.Practices.Prism;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;

namespace IgOutlook.Modules.Contacts
{
    public class ContactsViewBase : NavigationAwareViewModelBase
    {
        #region Properties

        protected IEventAggregator EventAggregator { get; private set; }
        protected IContactService ContactService { get; private set; }
        protected IDialogService DialogService { get; private set; }

        public DelegateCommand<string> ContactCommand { get; private set; }
        public DelegateCommand DeleteCommand { get; private set; }

        private Contact activeContact;
        public Contact ActiveContact
        {
            get { return activeContact; }
            set { activeContact = value; OnPropertyChanged(); }
        }

        #endregion //Properties

        #region Constructors

        public ContactsViewBase(IEventAggregator eventAggragator, IContactService contactService, IDialogService dialogService)
        {
            EventAggregator = eventAggragator;
            ContactService = contactService;
            DialogService = dialogService;

            ContactCommand = new DelegateCommand<string>(ExecuteConactCommand, CanExecuteConactCommand);
            DeleteCommand = new DelegateCommand(DeleteContact, HasValidContact);

            this.PropertyChanged += (s, a) =>
            {
                if (a.PropertyName == "ActiveContact")
                    DeleteCommand.RaiseCanExecuteChanged();
            };

            IconSource = "pack://application:,,,/IgOutlook.Infrastructure;component/Images/Contact.ico";
        }

        #endregion //Constructors

        #region Commands

        protected virtual void ExecuteConactCommand(string action)
        {
            string view = "ContactDetailsView";
            UriQuery parameter = null;

            if (action.Equals(ContactParameters.New))
                parameter = CreateContactParameter(ContactParameters.New);
            else if (action.Equals(ContactParameters.View))
            {
                if (ActiveContact == null) return;

                view = "ContactDetailsView";
                parameter = CreateContactParameter(ContactParameters.View);
            }

            DialogService.ShowRibbonDialog(string.Format("{0}{1}", view, parameter));
        }

        protected virtual bool CanExecuteConactCommand(string action)
        {
            return true;
        }

        protected virtual void DeleteContact()
        {
            ContactService.DeleteContact(ActiveContact);
        }

        protected virtual bool HasValidContact()
        {
            return ActiveContact != null;
        }

        #endregion //Commands

        #region Methods

        protected virtual UriQuery CreateContactParameter(string contactMode)
        {
            UriQuery query = new UriQuery();

            query.Add(ContactParameters.ContactModeKey, contactMode);

            var sourceConatcId = "NEW";

            if (!contactMode.Equals(ContactParameters.New))
            {
                sourceConatcId = activeContact.Id.ToString();
            }

            query.Add(ContactParameters.ContactIdKey, sourceConatcId.ToString());

            return query;
        }

        #endregion //Methods
    }
}
